package com.ozs.web.controller.websocket;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.utils.AppendUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.MsgAppPush;
import com.ozs.service.entity.MsgWebPush;
import com.ozs.service.entity.vo.MsgAlarmResp;
import com.ozs.service.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Author : sunhh
 * @create 2023/3/13 22:08
 */
@Slf4j
@Controller
@RequestMapping("/sdk")
public class WebSocketConteoller extends BaseController {

    @Autowired
    private WebSocketServer webSocketServer;
    @Autowired
    private BaseUserService baseUserService;
    @Resource
    private BaseRailwayManagementService baseRailwayManagementService;
    @Autowired
    private BaseCameraManagementService baseCameraManagementService;
    @Autowired
    private UserRegistrationidService userRegistrationidService;
    @Autowired
    private MsgWebPushService msgWebPushService;
    @Autowired
    private MsgAppPushService msgAppPushService;

    @ResponseBody
    @PostMapping("/publish")
    public Map publish(@RequestBody MsgAlarm msgAlarm) {
        LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<BaseCameraManagement>();
        if (!ObjectUtils.isEmpty(msgAlarm.getCameraCode())) {
            wrapper.eq(BaseCameraManagement::getCameraCode, msgAlarm.getCameraCode());
        }
        BaseCameraManagement baseCameraManagement = baseCameraManagementService.getOne(wrapper);
        // 获取推送人ID列表
        Set<String> set = baseUserService.getUserIdList(baseCameraManagement.getCreateBy(), baseCameraManagement.getDeptId());
        String[] list = set.toArray(new String[set.size()]);
        List<String> userIdList = new ArrayList<>(set);
        MsgAlarmResp msgAlarmResp = new MsgAlarmResp();
        String begin = AppendUtils.stringAppend(baseCameraManagement.getBeginMile());
        String end = AppendUtils.stringAppend(baseCameraManagement.getEndMile());
        String endMile = begin + "-" + end;
        msgAlarmResp.setBeginMile(endMile);
        msgAlarmResp.setAlarmType(msgAlarm.getAlarmType());
        String mile = AppendUtils.stringAppend(baseCameraManagement.getInstallMile());
        msgAlarmResp.setAlarmMile(mile);
        String cameraCode = baseCameraManagement.getCameraCode();
        String cameraName = baseRailwayManagementService.getCameraNameByCameraCode(cameraCode);
        msgAlarmResp.setRailwayName(cameraName != null ? cameraName : "");
        msgAlarmResp.setLineDirStr(baseCameraManagement.getLineDir() == 1 ? "上行" : "下行");
        msgAlarmResp.setAlarmTime(msgAlarm.getAlarmTime() != null ? msgAlarm.getAlarmTime() : new Date());
        // 推送消息reqMsgAlarmVo
        webSocketServer.sendMoreMessage(list, objStr(msgAlarmResp));
        // app 消息推送
        List<String> alias = userRegistrationidService.selectRegistrationID(userIdList);
        // List<String> alias = new ArrayList<String>();
        // alias.add("sunhh");
        Date alarmTime = msgAlarm.getAlarmTime();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String formatDate = sdf.format(alarmTime != null ? alarmTime : new Date());
        JPushUtil.sendToRegistrationId(
                alias,
                "报警通知！", cameraName != null ? cameraName : "",
                formatDate + "     " + (msgAlarmResp.getAlarmType() == 1 ? "泥石流" : msgAlarmResp.getAlarmType()) + "     " + (baseCameraManagement.getLineDir() == 1 ? "上行" : "下行") + "     " + mile + "     " + endMile,
                "");
        // 存入数据
        if (!StringUtils.isEmpty(msgAlarm.getAlarmId()) || !StringUtils.isEmpty(userIdList)) {
            String userIdLogin = getUserId();
            String alarmId = msgAlarm.getAlarmId();
            for (String userId : userIdList) {
                MsgWebPush msgWebPush = new MsgWebPush();
                // 状态：1未读；2已读
                msgWebPush.setStatus(1);
                msgWebPush.setAlarmId(alarmId);
                msgWebPush.setReceiveBy(userId);
                msgWebPush.setCreateBy(userIdLogin);
                msgWebPush.setCreateTime(new Date());
                msgWebPush.setUpdateBy(userIdLogin);
                msgWebPush.setUpdateTime(new Date());
                msgWebPushService.save(msgWebPush);
                MsgAppPush msgAppPush = new MsgAppPush();
                BeanUtils.copyProperties(msgWebPush, msgAppPush);
                msgAppPushService.save(msgAppPush);
            }
        }
        HashMap<String, Object> map = new HashMap<>();
        map.put("code", 200);
        return map;
    }

    private String objStr(MsgAlarmResp msgAlarmResp) {
        JSONObject obj = new JSONObject();
        obj.put("data", msgAlarmResp); //data
        obj.put("code", 110); //code
        log.info("【websocket消息】 单点消息:" + obj.toJSONString());
        return obj.toJSONString();
    }
}
